@@ -1,11 +1,6 @@
-/*
- * Copyright (C) 2007 Stig Venaas <venaas@uninett.no>
- * Copyright (C) 2010,2011 NORDUnet A/S
- *
- * Permission to use, copy, modify, and distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
- */
+/* Copyright (c) 2007-2009, UNINETT AS
+ * Copyright (c) 2010-2011, NORDUnet A/S */
+/* See LICENSE for licensing information. */
 
 #ifndef SYS_SOLARIS9
 #include <stdint.h>
@@ -23,6 +18,7 @@
 void debug_init(char *ident);
 void debug_set_level(uint8_t level);
 void debug_timestamp_on();
+void debug_tid_on();
 uint8_t debug_get_level();
 void debug(uint8_t level, char *format, ...);
 void debugx(int status, uint8_t level, char *format, ...);
@@ -30,9 +26,7 @@ void debugerrno(int err, uint8_t level, char *format, ...);
 void debugerrnox(int err, uint8_t level, char *format, ...);
 int debug_set_destination(char *dest, int log_type);
 void debug_reopen_log();
-#if defined(WANT_FTICKS)
 void fticks_debug(const char *format, ...);
-#endif
 
 /* Local Variables: */
 /* c-file-style: "stroustrup" */
