@@ -3,6 +3,7 @@
 /* See LICENSE for licensing information. */
 
 #include <sys/socket.h>
+#include <sys/time.h>
 #include <netdb.h>
 
 #define SOCKADDR_SIZE(addr) ((addr).ss_family == AF_INET ?	\
@@ -23,6 +24,7 @@ void disable_DF_bit(int socket, struct addrinfo *res);
 void enable_keepalive(int socket);
 int bindtoaddr(struct addrinfo *addrinfo, int family, int reuse);
 int connecttcp(struct addrinfo *addrinfo, struct addrinfo *src, uint16_t timeout);
+time_t connect_wait(struct timeval attempt_start, struct timeval last_success, int firsttry);
 
 
 /* Local Variables: */
